#!/usr/bin/env pybricks-micropython
from pybricks.hubs import EV3Brick
from pybricks.ev3devices import (Motor, TouchSensor, ColorSensor,
                                 InfraredSensor, UltrasonicSensor, GyroSensor)
from pybricks.parameters import Port, Stop, Direction, Button, Color
from pybricks.tools import wait, StopWatch, DataLog
from pybricks.robotics import DriveBase
from pybricks.media.ev3dev import SoundFile, ImageFile


# This program requires LEGO EV3 MicroPython v2.0 or higher.
# Click "Open user guide" on the EV3 extension tab for more information.


# Create your objects here.
ev3 = EV3Brick()

right_motor = Motor(Port.D, positive_direction=Direction.CLOCKWISE)
left_motor = Motor(Port.C, positive_direction=Direction.COUNTERCLOCKWISE)
motor_one = Motor(Port.B)

line_sensor = ColorSensor(Port.S4)
Color_plus = ColorSensor(Port.S1)
Straight_sensor = UltrasonicSensor(Port.S2)
Sensor_45 = UltrasonicSensor(Port.S3)

def stopping():
    right_motor.stop()
    left_motor.stop()

def motors(gradus, speed_left, speed_right):
    if gradus > 0:
        left_motor.reset_angle(0)
        right_motor.reset_angle(0)
        while True:
            if speed_right != 0:
                right_motor.run(speed_right)
            if speed_left != 0:
                left_motor.run(speed_left)
            if speed_left < 0 and speed_right > 0:
                if left_motor.angle() <= gradus*-1 and right_motor.angle() >= gradus:
                    right_motor.stop()
                    left_motor.stop()
                    return
            elif speed_left < 0 and speed_right < 0:
                if left_motor.angle() <= gradus*-1 and right_motor.angle() <= gradus*-1:
                    right_motor.stop()
                    left_motor.stop()
                    return
            elif speed_right < 0 and speed_left > 0:
                if right_motor.angle() <= gradus*-1 and left_motor.angle() >= gradus:
                    right_motor.stop()
                    left_motor.stop()
                    return
            elif speed_left > 0 and speed_right > 0:
                if left_motor.angle() >= gradus and right_motor.angle() >= gradus:
                    right_motor.stop()
                    left_motor.stop()
                    return
            elif speed_right > 0:
                if right_motor.angle() >= gradus:
                    right_motor.stop()
                    left_motor.stop()
                    return
            elif speed_right < 0:
                if right_motor.angle() <= gradus*-1:
                    right_motor.stop()
                    left_motor.stop()
                    return
            elif speed_left > 0:
                if left_motor.angle() >= gradus:
                    right_motor.stop()
                    left_motor.stop()
                    return
            elif speed_left < 0:
                if left_motor.angle() <= gradus*-1:
                    right_motor.stop()
                    left_motor.stop()
                    return
    else:
        right_motor.run(speed_right)
        left_motor.run(speed_left)

way = [0, 0, 0, -1, -1, 0, 1, 0, 0, 1, 0, -1, -1, 0, 1, 0, 1, 0, 0, 1, 0, 0, 1, 0, -1, -1, 0, 1, 0, -1, -1, 0, 0, 0, -1, 0, 0, 1, 0, 1, 0, 0]
#0 - Straight
#1 - right
#-1 - left

one = 0
two = 0
reverseColour = 0
ss = 0

colour_of_line = True

BLACK = 25
WHITE = 40
threshold = (BLACK + WHITE) / 2

# Set the drive speed at 100 millimeters per second.
PROPORTIONAL_GAIN = 2.1
robot = DriveBase(right_motor, left_motor, wheel_diameter=55.5, axle_track=155)

for i in way:
    if i == 0:
        left_motor.reset_angle(0)
        right_motor.reset_angle(0)
        while left_motor.angle() < 544 or right_motor.angle() < 544:
            if Sensor_45.distance() < 70 and Sensor_45.distance() < 150:
                motors(-1, 290, 340)
            elif Sensor_45.distance() > 70 and Sensor_45.distance() < 150:
                motors(-1, 340, 300)
            else:
                motors(-1, 300, 300)
    elif i == 1:
        motors(200, -200, 200)
    elif i == -1:
        motors(200, 200, -200)

#############

motors(200, 500, 500)

while Color_plus.reflection() > 10 or line_sensor.reflection() > 10:
    '''if line_sensor.reflection() > 20:
        motors(0, 125, 600)
    else:
        motors(0, 600, 125)'''
    robot.drive(130, (line_sensor.reflection()-threshold)*PROPORTIONAL_GAIN)
    if Straight_sensor.distance() < 150:
        robot.stop()
        motors(70, -900, -900)
        motors(150, -900, -400)
        wait(30)
        motors(130, 900, 900)
        motors(0, 800, 520)
        while line_sensor.reflection() > 20:
            pass
        motors(40, 300, 300)
        motors(90, -400, 400)

robot.stop()

motors(230, 400, 400)