#!/usr/bin/env pybricks-micropython
from pybricks.hubs import EV3Brick
from pybricks.ev3devices import (Motor, TouchSensor, ColorSensor,
                                 InfraredSensor, UltrasonicSensor, GyroSensor)
from pybricks.parameters import Port, Stop, Direction, Button, Color
from pybricks.tools import wait, StopWatch, DataLog
from pybricks.robotics import DriveBase
from pybricks.media.ev3dev import SoundFile, ImageFile


# This program requires LEGO EV3 MicroPython v2.0 or higher.
# Click "Open user guide" on the EV3 extension tab for more information.


# Create your objects here.
ev3 = EV3Brick()

colour_of_line = True

right_motor = Motor(Port.D, positive_direction=Direction.CLOCKWISE)
left_motor = Motor(Port.C, positive_direction=Direction.COUNTERCLOCKWISE)
motor_one = Motor(Port.B)

line_sensor = ColorSensor(Port.S4)
Color_plus = ColorSensor(Port.S1)
Straight_sensor = UltrasonicSensor(Port.S2)
Sensor_45 = UltrasonicSensor(Port.S3)

def stopping():
    right_motor.stop()
    left_motor.stop()

def motors(gradus, speed_left, speed_right):
    if gradus > 0:
        left_motor.reset_angle(0)
        right_motor.reset_angle(0)
        while True:
            if speed_right != 0:
                right_motor.run(speed_right)
            if speed_left != 0:
                left_motor.run(speed_left)
            if speed_left < 0 and speed_right > 0:
                if left_motor.angle() <= gradus*-1 and right_motor.angle() >= gradus:
                    right_motor.stop()
                    left_motor.stop()
                    return
            elif speed_left < 0 and speed_right < 0:
                if left_motor.angle() <= gradus*-1 and right_motor.angle() <= gradus*-1:
                    right_motor.stop()
                    left_motor.stop()
                    return
            elif speed_right < 0 and speed_left > 0:
                if right_motor.angle() <= gradus*-1 and left_motor.angle() >= gradus:
                    right_motor.stop()
                    left_motor.stop()
                    return
            elif speed_left > 0 and speed_right > 0:
                if left_motor.angle() >= gradus and right_motor.angle() >= gradus:
                    right_motor.stop()
                    left_motor.stop()
                    return
            elif speed_right > 0:
                if right_motor.angle() >= gradus:
                    right_motor.stop()
                    left_motor.stop()
                    return
            elif speed_right < 0:
                if right_motor.angle() <= gradus*-1:
                    right_motor.stop()
                    left_motor.stop()
                    return
            elif speed_left > 0:
                if left_motor.angle() >= gradus:
                    right_motor.stop()
                    left_motor.stop()
                    return
            elif speed_left < 0:
                if left_motor.angle() <= gradus*-1:
                    right_motor.stop()
                    left_motor.stop()
                    return
    else:
        right_motor.run(speed_right)
        left_motor.run(speed_left)


way = []
#0 - Straight
#1 - right
#-1 - left

one = 0
two = 0

BLACK = 25
WHITE = 55
threshold = (BLACK + WHITE) / 2

# Set the drive speed at 100 millimeters per second.

dist = 0
DRIVE_SPEED = 100
PROPORTIONAL_GAIN = 2.1
robot = DriveBase(right_motor, left_motor, wheel_diameter=55.5, axle_track=155)

while line_sensor.reflection() > 15:
    if Sensor_45.distance() > 200:
        robot.stop()
        motors(193, 200, -200)
        left_motor.reset_angle(0)
        right_motor.reset_angle(0)
        while left_motor.angle() < 544 or right_motor.angle() < 544:
            if Sensor_45.distance() > 70 and Sensor_45.distance() < 150:
                motors(-1, 300, 270)
            elif Sensor_45.distance() < 70 and Sensor_45.distance() < 150:
                motors(-1, 270, 300)
            else:
                motors(-1, 300, 300)
        robot.stop()
        way.append(1)
        way.append(0)
    elif Straight_sensor.distance() > 150:
        robot.stop()
        left_motor.reset_angle(0)
        right_motor.reset_angle(0)
        while left_motor.angle() < 541 or right_motor.angle() < 541:
            if Sensor_45.distance() < 70 and Sensor_45.distance() < 150:
                motors(-1, 300, 340)
            elif Sensor_45.distance() > 70 and Sensor_45.distance() < 150:
                motors(-1, 340, 300)
            else:
                motors(-1, 300, 300)
        robot.stop()
        way.append(0)
    else:
        robot.stop()
        motors(203, -200, 200)
        way.append(-1)
    ev3.screen.print(way)
    if len(way) == 41:
        motors(75, 0, 200)
        while line_sensor.reflection() > 10:
            if Sensor_45.distance() < 50 and Sensor_45.distance() < 150:
                motors(-1, 300, 340)
            elif Sensor_45.distance() > 50 and Sensor_45.distance() < 150:
                motors(-1, 340, 300)
            else:
                motors(-1, 300, 300)
        motors(200, 200, 200)
        break
    if len(way) == 34:
        motors(40, -250, -250)
    if len(way) == 31:
        wait(500)
    wait(300)

motors(200, 400, 400)

for i in range(len(way)):
    if way[i] == -1 and len(way) > i:
        if way[i+1] == -1:
            one = i
            two = i+1
            way[one] = None
            way[two] = None
            while True:
                if way[one-1] == None:
                    one -= 1
                elif way[two+1] == None:
                    two += 1
                else:
                    if way[one-1] + way[two+1] == 0:
                        way[two+1] = None
                        way[one-1] = None
                    elif way[one-1] + way[two+1] == 1:
                        way[two+1] = -1
                        break
                    elif way[one-1] + way[two+1] == 2:
                        way[one-1] = None
                        way[two+1] = None
                        break
                    one -= 1
                    two += 1

way.reverse()

while not (Button.CENTER in ev3.buttons.pressed()):
    pass

for i in way:
    if i == 0:
        left_motor.reset_angle(0)
        right_motor.reset_angle(0)
        while left_motor.angle() < 544 or right_motor.angle() < 544:
            if Sensor_45.distance() < 70 and Sensor_45.distance() < 150:
                motors(-1, 290, 340)
            elif Sensor_45.distance() > 70 and Sensor_45.distance() < 150:
                motors(-1, 340, 300)
            else:
                motors(-1, 300, 300)
    elif i == 1:
        motors(200, -200, 200)
    elif i == -1:
        motors(200, 200, -200)