#!/usr/bin/env pybricks-micropython
from pybricks.hubs import EV3Brick
from pybricks.ev3devices import (Motor, TouchSensor, ColorSensor,
                                 InfraredSensor, UltrasonicSensor, GyroSensor)
from pybricks.parameters import Port, Stop, Direction, Button, Color
from pybricks.tools import wait, StopWatch, DataLog
from pybricks.robotics import DriveBase
from pybricks.media.ev3dev import SoundFile, ImageFile


# This program requires LEGO EV3 MicroPython v2.0 or higher.
# Click "Open user guide" on the EV3 extension tab for more information.


# Create your objects here.
ev3 = EV3Brick()

right_motor = Motor(Port.D, positive_direction=Direction.CLOCKWISE)
left_motor = Motor(Port.C, positive_direction=Direction.COUNTERCLOCKWISE)
motor_one = Motor(Port.B)

line_sensor = ColorSensor(Port.S4)
Color_plus = ColorSensor(Port.S1)
Straight_sensor = UltrasonicSensor(Port.S2)
Sensor_45 = UltrasonicSensor(Port.S3)

def stopping():
    right_motor.stop()
    left_motor.stop()

def motors(gradus, speed_left, speed_right):
    if gradus > 0:
        left_motor.reset_angle(0)
        right_motor.reset_angle(0)
        while True:
            if speed_right != 0:
                right_motor.run(speed_right)
            if speed_left != 0:
                left_motor.run(speed_left)
            if speed_left < 0 and speed_right > 0:
                if left_motor.angle() <= gradus*-1 and right_motor.angle() >= gradus:
                    right_motor.stop()
                    left_motor.stop()
                    return
            elif speed_left < 0 and speed_right < 0:
                if left_motor.angle() <= gradus*-1 and right_motor.angle() <= gradus*-1:
                    right_motor.stop()
                    left_motor.stop()
                    return
            elif speed_right < 0 and speed_left > 0:
                if right_motor.angle() <= gradus*-1 and left_motor.angle() >= gradus:
                    right_motor.stop()
                    left_motor.stop()
                    return
            elif speed_left > 0 and speed_right > 0:
                if left_motor.angle() >= gradus and right_motor.angle() >= gradus:
                    right_motor.stop()
                    left_motor.stop()
                    return
            elif speed_right > 0:
                if right_motor.angle() >= gradus:
                    right_motor.stop()
                    left_motor.stop()
                    return
            elif speed_right < 0:
                if right_motor.angle() <= gradus*-1:
                    right_motor.stop()
                    left_motor.stop()
                    return
            elif speed_left > 0:
                if left_motor.angle() >= gradus:
                    right_motor.stop()
                    left_motor.stop()
                    return
            elif speed_left < 0:
                if left_motor.angle() <= gradus*-1:
                    right_motor.stop()
                    left_motor.stop()
                    return
    else:
        right_motor.run(speed_right)
        left_motor.run(speed_left)

way = []
#0 - Straight
#1 - right
#-1 - left

one = 0
two = 0
reverseColour = 0
ss = 0

colour_of_line = True

BLACK = 25
WHITE = 40
threshold = (BLACK + WHITE) / 2
threshold_2 = (15 + WHITE) / 2


# Set the drive speed at 100 millimeters per second.
PROPORTIONAL_GAIN = 2.4
robot = DriveBase(right_motor, left_motor, wheel_diameter=55.5, axle_track=155)

motors(350, 500, 500)

while Color_plus.reflection() > 15 or line_sensor.reflection() > 15:
    '''if line_sensor.reflection() > 20:
        motors(0, 125, 600)
    else:
        motors(0, 600, 125)'''
    robot.drive(180, (line_sensor.reflection()-threshold_2)*2.8)
    if Straight_sensor.distance() < 150:
        robot.stop()
        motors(80, -900, -900)
        motors(150, -900, -350)
        wait(30)
        motors(170, 900, 900)
        motors(0, 800, 520)
        while line_sensor.reflection() > 20:
            pass
        motors(60, 300, 300)
        motors(90, -400, 400)

robot.stop()

motors(30, 150, 150)

while not (Sensor_45.distance() in [65, 66, 67, 68, 69, 70 , 71, 72, 73, 74]):
    if Sensor_45.distance() > 70:
        motors(-1, 150, 0)
    elif Sensor_45.distance() < 70:
        motors(-1, 0, 150)

motors(310, 500, 500)

################

while line_sensor.reflection() > 15:
    if Sensor_45.distance() > 200:
        robot.stop()
        motors(190, 200, -200)
        left_motor.reset_angle(0)
        right_motor.reset_angle(0)
        while left_motor.angle() < 542 or right_motor.angle() < 542:
            if Sensor_45.distance() > 70 and Sensor_45.distance() < 150:
                motors(-1, 300, 270)
            elif Sensor_45.distance() < 70 and Sensor_45.distance() < 150:
                motors(-1, 270, 300)
            else:
                motors(-1, 300, 300)
        robot.stop()
        way.append(1)
        way.append(0)
    elif Straight_sensor.distance() > 150:
        robot.stop()
        left_motor.reset_angle(0)
        right_motor.reset_angle(0)
        while left_motor.angle() < 539 or right_motor.angle() < 539:
            if Sensor_45.distance() < 70 and Sensor_45.distance() < 150:
                motors(-1, 300, 340)
            elif Sensor_45.distance() > 70 and Sensor_45.distance() < 150:
                motors(-1, 340, 300)
            else:
                motors(-1, 300, 300)
        robot.stop()
        way.append(0)
    else:
        robot.stop()
        motors(205, -200, 200)
        way.append(-1)
    ev3.screen.print(way)
    if len(way) == 41:
        while line_sensor.reflection() > 10:
            if Sensor_45.distance() < 50 and Sensor_45.distance() < 150:
                motors(-1, 300, 340)
            elif Sensor_45.distance() > 50 and Sensor_45.distance() < 150:
                motors(-1, 340, 300)
            else:
                motors(-1, 300, 300)
        motors(200, 200, 200)
        break
    if len(way) == 34:
        motors(45, -250, -250)
    if len(way) == 31:
        wait(500)
    wait(300)

motors(200, 400, 400)

for i in range(len(way)):
    if way[i] == -1 and len(way) > i:
        if way[i+1] == -1:
            one = i
            two = i+1
            way[one] = None
            way[two] = None
            while True:
                if way[one-1] == None:
                    one -= 1
                elif way[two+1] == None:
                    two += 1
                else:
                    if way[one-1] + way[two+1] == 0:
                        way[two+1] = None
                        way[one-1] = None
                    elif way[one-1] + way[two+1] == 1:
                        way[two+1] = -1
                        break
                    elif way[one-1] + way[two+1] == 2:
                        way[one-1] = None
                        way[two+1] = None
                        break
                    one -= 1
                    two += 1

way.reverse()

###################

reverseColour = 0

while True:#инверсия
    while colour_of_line == True:
        if reverseColour == 10:
            robot.drive(100, (line_sensor.reflection() - threshold_2)*2.1)
        elif reverseColour == 1:
            robot.drive(100, (line_sensor.reflection() - threshold_2)*2.1)
        else:
            robot.drive(150, (line_sensor.reflection() - threshold_2)*2.1)
        if Color_plus.reflection() < 20:
            robot.stop()
            motors(5, 150, 150)
            colour_of_line = False
    reverseColour += 1
    if reverseColour == 15:
        break
    while colour_of_line == False:
        robot.drive(150, (threshold_2 - line_sensor.reflection())*2.1)
        if Color_plus.reflection() > 30:
            robot.stop()
            motors(5, 150, 150)
            colour_of_line = True
    reverseColour += 1

#########################


motors(900, 500, 500)
motors(500, 0, 200)
robot.stop()
motors(440, 0, -200)
motors(750, 400, 400)
motor_one.run_angle(300, 190, wait=True)
robot.stop()
motors(35, 0, 200)
while line_sensor.reflection() > 20:
    motors(-1, 550, 550)
robot.stop()
motors(700, -550, -550)
robot.stop()

for i in range(4):
    robot.turn(36)
    robot.stop()
    while line_sensor.reflection() > 20:
        motors(-1, 550, 550)
    robot.stop()
    motors(1100, -550, -550)

robot.turn(-20)
robot.stop()
motors(2250, 500, 500)

motors(220, -200, 200)
while line_sensor.reflection() > 15:
    motors(-1, 400, 400)

motors(100, -200, -200)

motors(400, 300, 0)

###################

reverseColour = 0
colour_of_line = True

while True:#инверсия
    while colour_of_line == True:
        robot.drive(150, (line_sensor.reflection() - threshold_2)*2.5)
        if Color_plus.reflection() < 20:
            robot.stop()
            motors(5, 150, 150)
            colour_of_line = False
    reverseColour += 1
    if reverseColour == 15:
        break
    while colour_of_line == False:
        robot.drive(150, (threshold_2 - line_sensor.reflection())*2.5)
        if Color_plus.reflection() > 30:
            robot.stop()
            motors(5, 150, 150)
            colour_of_line = True
    reverseColour += 1

##############

motors(280, 400, 400)

for i in way:
    if i == 0:
        left_motor.reset_angle(0)
        right_motor.reset_angle(0)
        while left_motor.angle() < 544 or right_motor.angle() < 544:
            if Sensor_45.distance() < 70 and Sensor_45.distance() < 150:
                motors(-1, 290, 340)
            elif Sensor_45.distance() > 70 and Sensor_45.distance() < 150:
                motors(-1, 340, 300)
            else:
                motors(-1, 300, 300)
    elif i == 1:
        motors(200, -200, 200)
    elif i == -1:
        motors(200, 200, -200)

#############

motors(200, 500, 500)

while Color_plus.reflection() > 10 or line_sensor.reflection() > 10:
    '''if line_sensor.reflection() > 20:
        motors(0, 125, 600)
    else:
        motors(0, 600, 125)'''
    robot.drive(130, (line_sensor.reflection()-threshold)*2.8)
    if Straight_sensor.distance() < 150:
        robot.stop()
        motors(70, -900, -900)
        motors(150, -900, -400)
        wait(30)
        motors(130, 900, 900)
        motors(0, 800, 520)
        while line_sensor.reflection() > 20:
            pass
        motors(40, 300, 300)
        motors(90, -400, 400)

robot.stop()

motors(230, 400, 400)